%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                 Basic Matlab Tutorial for CIS Students                  %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Matlab is a interpreting programming language that supports all basic 
% operations and functions like other programming languages, e.g. 
% if-then-else structure, case-switch structure, for loop, while loop, 
% break, try, catch, return, ...

% Unlike other languages matlab DOES NOT make executable code by default. 
% Instead, it INTERPRETS comands written in a M-script file or a function. 
% If necessary, e.g. for better performance, a matlab program can be 
% compiled into binary executable code. However, you will need a special
% toolbox, the matlab compiler. 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                            1. Data types                                % 
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% A typical matlab program starts with the CLEAR command, to clear all 
% variables from the workspace (right now there aren't any). 


% Delete all variables in scope
clear 


% Scalar
t = 10
f = 2.112
e = 1e-10
c = 'A'

% As in other scripting languages, there is NO need to EXPLICITLY declare
% variables. The data type of a variable is determined by the latest value
% assigned to it.


% Vector (or 1-D Array)
R = [1.2 2.4 4.8 9.6]    % row vector
C = [1.2                 % column vector
     2.4 
     4.8 
     9.6]
  
% How to access an element in the vector
V3 = C(3)     % 3rd element in C
  
% String (actually a row vector)
S = 'hello!' 
% or equivalently
S = ['h' 'e' 'l' 'l' 'o' '!']
% or even
S = char([104 101 108 108 111 33])  

% Matrix (or 2-D array)
M = [1  2  3              % a 4 by 3 matrix
     4  5  6
     7  8  9
     10 11 12]

% How to access an element 
E = M(2, 3)
M(2,3) = 99

% How to extract a row or a column
M_row = M(2, :)     % 2nd row
M_col = M(:, 3)     % 3rd column

% Matlab is very powerful in matrix manipulation/computation, we will come
% back to this shortly

% Structure
st.name = 'Peter';
st.age = 25;
st.major = 'CIS';
st.GPA = 3.96;
st

st(2).name = 'George';
st(2).age = 26;
st(2).major = 'MBA';
st(2).GPA = 3.00;

st(1).major

st(2)

st.name

% Cell array
ca{1} = t;
ca{2} = S;
ca{3} = st;
ca
ca(2:3)
ca{3}

% Let's see what we have in our workspace (main memory) by now
who
whos    

% pause


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                       2. Flow control structures                        %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Sequential 
a = 10
b = 33

% Branch (if-then-else, switch~case)
if a == 4
   b = a ^ 2 
elseif a ~= 10
   b = a
else
   b = -a
end

% Loop (for-loop, while-loop) 
for i = 1 : 2 : 10
	i   
end

% Function 
% A function can have more than one input and output arguments. Normally, 
% arguments are transfered by value. It is possible to submit arguments by
% reference (as common attributes for caller and the function), but you'll
% never need this...

a = 5
b = 4
[sum_ab, product_ab] = operations(a, b)

% In addition, Matlab has integrated powerful functons for elementary matrix
% manipulation and numerical linear algebra, elementary and specialized math
% functions, 2D and 3D graphic,  interpolation,  polynomials,  Fourier 
% transforms, sparse matrices, .....

% Matlab Help System (through help window, or through help command)
help mean

% There is also a lookfor statement, when we are not sure what is the exact 
% name of the statement or the function. Lookfor searches for specifief word
% through description of all available Matlab functions e.g. "lookfor write"
% gives all functions that contains the word write in their description


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                      3. Vector / Matrix operations                      %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Matlab is very powerful in matrix computation. Avoid using explicit loops
% for computing with vectors/matrices. Always try matlab built-in vector / 
% matrix operations first since they are optimized in Matlab.

% Two 4-by-3 matrices
A = [1  2  3      
     4  5  6
     7  8  9
     10 11 12]
B = [1  3  5
     7  11  13
     17 19  23
     29 31 37]
   
% Another way to define a matrix (3-by-4)  
C = [1 2 3 4; 5 6 7 8; 9 10 11 12]

D = A + B
E = B * C

% Multiply or divide by elements
F = A .* B
G = A ./ B

% Inverse
H = inv(E)

% Transpose
A
T = A'
T = transpose(A)

% multiply by scalar
Amult = A*3
% add a scalar
Aadd = A+5

% concatenate matrices (dimensions have to match)

A_B1 = [A B] % (number of rows must match)
A_B2 = [A; B] % (number of columns must match)
A_B3 = [A(:,1) B(:,[2 3])] % concatenate parts of matrices

% change a row
A_B1(3,:) = [1 1 1 1 1 1]

% change a column - multiply by a factor
A_B1(:,3) = A_B1(:,3) * 3

% delete rows
A_B2
A_B2(2:3,:) = []

% delete columns
A_B1
A_B1(:,2:5) = []

% Special matrices
W = zeros(4)         % 4-by-4 all zeros
W = zeros(4, 2)      % 4-by-2 all zeros
X = ones(4, 2)       % 4-by-2 all ones
Y = eye(4);          % 4-by-4 identity matrix
Z = diag([4 3 2 1])  % diagonal matrix

% Obtaining dimension information
A
[num_rows, num_cols] = size(A)
num_rows = size(A, 1)
num_cols = size(A, 2)

% Sumation 
A
sum(A)       % for each column
sum(A, 1)    % also for each column
sum(A, 2)    % for each row (the second index)
sum(sum(A))  % over all elements

% Basic Statistics
A
mean_A_col = mean(A)         % mean for each column
mean_A_row = mean(A, 2)      % mean for each row
std_A_col = std(A)           % standard devaition for each column
std_A_row = std(A, 0, 2)     % standard deviation for each row (the second parameter '0' MUST be present)
median_A_col = median(A, 1)  % median for each column
median_A_row = median(A, 2)  % median for each row

% Sorting
a = [0     7     1     5     0
     1     7     4     6     1
     4     3     9     1     5
     8     9     4     5     0
     9     3     3     5     2
     0     1     4     9     3
     7     7     7     9     9
     8    10     7     5     3
     7     2     9     8     3
    10     4     8     7     4]

% SORT each column separately
a1 = sort(a)       

% SORT each row separately
a2 = sort(a, 2)    

% SORT rows in ascending order according to columns 1, 2, 3, 4, 5
a3 = sortrows(a)

% SORT rows in ascending order according to columns 3 and 5
a4 = sortrows(a, [3 5])

% trick to SORT columns

a5 = sortrows(a')'

% FIND all rows (examples) where the values in second columen are greater than 5
a
idx = find(a(:, 2) > 5)

% print these rows
a_found = a(idx, :)

% another way to the same thing
a_foud1 = a(find(a(:,2) > 5), :)

% Special matrix functions
diagonal_matrix = diag([1 2 3 4])
all_zeros = zeros(3, 4)
all_ones = ones(4, 2)
identity_matrix = eye(4)
rand_matrix = rand(4, 3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                 4. Random Number Generation and Sampling                %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Always INITIALIZE the random number generator first
rand('state', sum(100 * clock))

% Generating random numbers from a UNIFORM distribution in the interval (0, 1)
R1 = rand            % a single number
R2 = rand(10, 3)     % a matrix of 10*3 numbers

% Generating random numbers from a UNIVARIATE NORMAL distribution of mean 0 
% and variance 1
R3 = randn           % a single number
R3 = randn(10, 3)    % a matrix of 10*3 numbers

% Generating examples from a MULTIVARIATE NORMAL distribution
mu = [2 3];                       % mean vector 
sigma = [1.2 1.5; 1.5 3];         % covariance matrix
data = mvnrnd(mu, sigma, 5000);   % total 5000 examples

figure(1)                         % visualize it
hold off
plot(data(:,1), data(:,2), '.')   
hold on
line([-3 2], [3 3], 'Color', 'r', 'LineStyle', ':');
line([2 2], [-4 3], 'Color', 'r', 'LineStyle', ':');
plot(2, 3, 'r.')
hold off

% Now we want to randomly select 500 examples from "data" 

% A. random sampling WITH replacement (an example might be selected more than
% once)
N = size(data, 1);                        % available number of examples   
idx = ceil(N * rand(500, 1));  % generate an index array, i.e. 500 
                                          % random integers between 1 and N
sdata = data(idx, :);                     % obtain the selected examples

% B. random sampling WITHOUT replacement (an example can not be selected more
% than once)
N = size(data, 1);                        % available number of examples   
idx = randperm(N);                        % randomly permute the indices of 1-N
idx = idx(1 : 500);                       % keep the first 500 indices
sdata = data(idx, :);                     % obtain the selected examples

% histogram example
hist(data(:,1))
hist(data(:,1),20) % 20 bins instead of 10

% compare distributions of 1st column for whole dataset and the sample
boxplot([ data(:,1) ; sdata(:,1) ], [ ones(size(data,1),1)*1 ; ones(size(sdata,1),1)*2 ])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                            5. Input / output                            %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Use semicolon to suppress display, compare
A = 1
% and
A = 1;

% To show a text message
disp('Hello, world!');
% or
disp 'Hello, world!'

% To show formatted text messages
for i = 1 : 3
    disp(sprintf('Iteration %d\n', i));
    % Or
    % fprintf(1, 'Iteration %d\n', i);   % print to standard output (file 1)
end

% Save specified variables into a specified file in ascii format
E
save ascii_matrix.txt E -ASCII 

% Save specified variables into a specified file in binary (matlab-specific) 
% format. The default extension is *.mat
save bin_matrices A B C

% To save ALL variables, write only a file name. All variables in scope will 
% be saved
whos
save binary_all

% How to load results from a *.mat file
clear
load binary_all 
whos

% From an ASCII file
E_matrix = load('ascii_matrix.txt')
E

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To read from general binary files, functions fopen, fread etc. have to  %
% be used. Following code reads and parses the ADULT data from UCI Machine%
% Learning Repository                                                     %     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

% open the file in read-only mode
f = fopen('adult.data', 'rb');

% initialize the data matrix
data = [];

% # of examples(lines) read
n1 = 0;
% # of examples with missing values
n2 = 0;

% read in the first example(line)
ln = fgetl(f);

% at the end of file, fgetl will return -1 (EOF)
while ischar(ln) & isempty(ln) == 0  
    
    % if n1 == 5000
    %     break;
    % end

    % if there are missing values (?), ignore current example. 
    if isempty(find(ln == '?'))
        
        % break down the current line into fields (in a STRING array)
        a = strread(ln, '%s', 'delimiter', ',');
        
        % convert the attributes (only the first three continuous ones are shown)
        age = str2num(a{1});
        fnlwgt = str2num(a{3});
        education_num = str2num(a{5});
        
        % convert the target variable
        if strcmp(a{15}, '>50K')
            target = 1;
        else
            target = 0;
        end
        
        % append it to data
        data = [data; age fnlwgt education_num target];
        
    else
        % at least one missing values
        n2 = n2 + 1;
    end
    
    % display progress
    n1 = n1 + 1;
    if mod(n1, 1000) == 0
        disp(sprintf('%d records read ...', n1));
    end
    
    % read in next example(line) 
    ln = fgetl(f);
end

% close the file
fclose(f);

disp(sprintf('%d examples read, %d examples with missing values', n1, n2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Ececute an external command
!copy ascii_matrix.txt copy_of_ascii_matrix.txt

% delete that file
delete('copy_of_ascii_matrix.txt');

% now, make it again using command 'dos' (for unix systems use 'unix')
dos('copy ascii_matrix.txt copy_of_ascii_matrix.txt');

% Eval command, useful in many cases (here just to delete a file, system command again)
file_to_delete = 'copy_of_ascii_matrix.txt';
eval(['! del ' file_to_delete]);
% eval(sprintf('! del %s', file_to_delete));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         % 
%                              6. Graphics                                %
%                                                                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% 1-D curve
t = 0 : 0.1 : 5
x = sin(pi * t) + 0.1 * t
y = cos(pi * t) - 0.2 * t
plot(t, x);


% 2-D curve
plot(x, y);
hold on
plot(x, y, 'r*');       % plots red stars instead of line
hold off

% 3-D curve
figure;
z =2 * t;
plot3(x, y, z, 'm-d');  % shows magenta diamonds in addition to line

% 3-D surface
[X, Y, Z] = peaks(30);    % a sample function of two variables. 

% 3-D shaded surface plot
figure(1)
surfc(X, Y, Z)

% pseudocolor plot
figure(2)
pcolor(X, Y, Z)
colormap(gray)
colormap(hsv)
colormap(copper)
help gray
colormap(hot)



