%********************************************************************
%                                                                   * 
%                       Data Mining Examples                        *
%                                                                   * 
%********************************************************************

%********************************************************************
%     CLASSIFICATION PROBLEM - PREDICT CLASS LABELS (DISCRETE)      *
%                  A Logistic Regression Example                    *
%********************************************************************

clear

% Pima Indians Diabetes Database
%   ftp://ftp.ics.uci.edu/pub/machine-learning-databases/pima-indians-diabetes/
% Description in file "pima.names", data in file "pima.data"

% The goal: to classify patients into 2 classes, i.e. whether a patient shows signs of diabetes, based on 8 features

% load the dataset (8 features, 1 class label 1/2)
pima = load('pima.data');
[nr, nc] = size(pima)
nf = nc - 1;      % number of features

% Split the data into training (70%) and test subsets (30%), no need for validation set in this case
[ptrain, ptest] = divideset(pima, 30);     

% normalize the training set (not including the target variable!)
[meanv, stdv, ptrain(:, 1 : nf)] = normalize(ptrain(:, 1 : nf), [], []);

% if some features are correlated, we have to do Principal Component Analysis (PCA) first to de-correlate the features

% add a column of ones into the training set
ptrain_c = ptrain(:, nf + 1);
ptrain = [ptrain(:, 1 : nf) ones(size(ptrain, 1), 1)];

% train Logistic Regression model
LR_model = logit(ptrain_c, ptrain);

% normalize test set using 'meanv' and 'stdv' calculated from the training set
[t1, t2, ptest(:, 1 : nf)] = normalize(ptest(:, 1 : nf), meanv, stdv);
% add a column of ones into the test set
ptest_c = ptest(:, nf + 1);
ptest = [ptest(:, 1 : nf) ones(size(ptest, 1), 1)];

% make prediction on the test set
prediction = ptest * LR_model.beta;
% transform results to 0-1 interval (logsig function can be also used)
prediction = 1 ./ (1 + exp(-prediction));

% evaluate accuracy of prediction
TN = sum(prediction < 0.5 & ptest_c == 0); % predicted class 0, true class 0
FN = sum(prediction < 0.5 & ptest_c == 1); % predicted class 0, true class 1
FP = sum(prediction >= 0.5 & ptest_c == 0); % predicted class 1, true class 0
TP = sum(prediction >= 0.5 & ptest_c == 1); % predicted class 1, true class 1
accuracy1 = (TN + TP) / (TP+TN+FP+FN)

% maybe, a better way to calculate accuracy is for both classes
sensitivity = TP / (TP+FN) % accuracy on class 1
specificity = TN / (TN+FP) % accuracy on class 0
accuracy2 = (sensitivity + specificity) / 2

% Question: if the accuracy is calculated by the last formula, is there a better way to build a model?
% Consider changing numbers of examples from each class.

disp 'done'
